﻿local Settings = LibStub:NewLibrary("SomeTracker-Settings", 1)
if not Settings then return end

local Templates = LibStub:GetLibrary("SomeTemplates-0.2")

local D = LibStub("DoomCore-0.6")
local K = D.K
local SharedMedia = LibStub( "LibSharedMedia-3.0" )


------------
-- Defaults
------------

local defaults = {}
defaults.db = {
  profile = { }
}
defaults.options = {
  type = "group",
  get = "ConfGet",
  set = "ConfSet",
  args = {
    add = {
      type = "input",
      name = "Add tracker",
      set = "AddTracker",
      width = "full",
      order = 0
    }
  }
}
defaults.tracker = {
  enabled = true,
  trackerType = "damage",
  markCrits = false,
  color = { r=255, g=255, b=255, a=1 },
  overrideColor = false,
  outline = "shadow",
  duration = "3",
  over = "exclude",
  lock = true,
  anchor = "center",
  offsetX = -210,
  offsetY = 100,
  iconSize = 31,
  font = "Accidental Presidency",
  fontSize = 22,
  add = { meterType = "spell" },
}
defaults.meter = {
  --image = "Interface\\Icons\\INV_Misc_QuestionMark",
  display = "normal",
  --color = conf:ConfGet(key:s(1):a("color")),
  color = {r=1,g=1,b=1,a=1},
  meterType = "spell"
}

Settings.defaults = defaults

local meterTypes = {
  aura = "Aura",
  pet = "Pet",
  spell = "Spell"
}
Settings.meterTypes = meterTypes
local outlines = {
  none = "None",
  outline = "Outline",
  shadow = "Shadow"
}
local trackerTypes = {
  damage = "Damage",
  healing = "Healing"
}

Settings.fromItem = {}
Settings.fromOther = {}
Settings.fromSpellbook = {}
Settings.fromPvp = {}


-----------------
-- Settings page
-----------------

local browsing
function Settings:BuildTrackerSettings(trackerName, tracker)
  local name = trackerName
  local opts = {
    type = "group",
    name = name,
    args = {
      rename = {
        type = "input",
        name = "Rename",
        get = "GetParentName",
        set = "RenameTracker",
        width = "full",
        order = 2,
      },
      enabled = {
        type  = "toggle",
        name = "Enabled",
        width = "half",
        order = 3
      },
      trackerType = {
        type = "select",
        name = "Type",
        values = trackerTypes,
        width = "half",
        order = 4
      },
      space0 = {
        type = "description",
        name = "",
        width = "full",
        order = 5
      },
      duration = {
        type = "input",
        name = "Duration (s)",
        desc = "Time in seconds before alerts fade",
        validate = "Number",
        width = "half",
        order = 10
      },
      over = {
        type = "select",
        name = "Overflow",
        desc = "Overdamage/Overhealing",
        width = "half",
        values = {
          include = "Include",
          exclude = "Exclude",
          additional = "Separate"
        },
        order = 11
      },
      markCrits = {
        type = "toggle",
        name = "Mark crits",
        set = "TrackerOverride",
        order = 12
      },
      space1 = {
        type = "description",
        name = "",
        width = "full",
        order = 13
      },
      color = {
        type = "color",
        name = "Color",
        width = "half",
        order = 20
      },
      overrideColor = {
        type = "toggle",
        name = "Override",
        width = "half",
        hidden = true,
        order = 21
      },
      space2 = {
        type = "description",
        name = "",
        width = "half",
        order = 22
      },
      outline = {
        type = "select",
        name = "Outline",
        values = outlines,
        width = "half",
        order = 23
      },
      space3 = {
        type = "description",
        name = "",
        width = "full",
        order = 24
      },
      iconSize = {
        type = "input",
        name = "Icon size",
        validate = "Number",
        set = "UpdateFrame",
        width = "half",
        order = 30
      },
      font = {
        name = "Font",
        type = "select",
        dialogControl = "LSM30_Font",
        values = SharedMedia:HashTable(SharedMedia.MediaType.FONT ),
        set = "UpdateFrame",
        order = 31
      },
      fontSize = {
        type = "input",
        name = "Font size",
        validate = "Number",
        set = "UpdateFrame",
        width = "half",
        order = 32
      },
      space4 = {
        type = "description",
        name = "",
        width = "full",
        order = 33
      },
      lock = {
        type = "toggle",
        name = "Lock",
        set = "UpdateFrame",
        width = "full",
        order = 40
      },
      anchor = {
        type = "select",
        name = "Anchor",
        set = "UpdateFrame",
        values = D.anchors,
        disabled = "Locked",
        order = 51
      },
      offsetX = {
        type = "input",
        name = "X",
        validate = "Number",
        set = "UpdateFrame",
        width = "half",
        disabled = "Locked",
        order = 52
      },
      offsetY = {
        type = "input",
        name = "Y",
        validate = "Number",
        set = "UpdateFrame",
        width = "half",
        disabled = "Locked",
        order = 53
      },
      delete = {
        type = "execute",
        name = "Delete",
        func = "DeleteTracker",
        order = 60
      },
      add = {
        type = "group",
        name = "Add to tracker",
        args = {
          meterType = {
            type = "select",
            name = "Type",
            width = "half",
            values = meterTypes,
            get = function(info)
              browsing = name
              return self.handler:ConfGet(info)
            end,
            order = 0
          },
          meterName = {
            type = "input",
            name = function() return "New " .. (tracker.meterType or "spell") end,
            set = "AddMeter",
            order = 1
          },
          fromItem = {
            type = "group",
            name = "From Items",
            inline = true,
            args = self.fromItem,
            hidden = function() return not next(self.fromItem) end,
            order = 2
          },
          fromOther = {
            type = "group",
            name = "From Other",
            inline = true,
            args = self.fromOther,
            hidden = function() return not next(self.fromOther) end,
            order = 10
          },
          fromSpellbook = {
            type = "group",
            name = "From Spellbook",
            inline = true,
            args = self.fromSpellbook,
            hidden = function() return not next(self.fromSpellbook) end,
            order = 4
          },
          fromPvp = {
            type = "group",
            name = "From Honor Talents",
            inline = true,
            args = self.fromPvp,
            hidden = function() return not next(self.fromPvp) end,
            order = 20
          }
        }
      }
    },
    order = 100 + D.orderNum(name)
  }
  if name == "Default Damage" or name == "Default Healing" then
    opts.args.delete = nil
    opts.args.rename = nil
  end
  for meterType, meterName, meter in self.handler:GetAll(tracker,"meter",3) do
    self:BuildMeterSettings(trackerName, meterType, meterName, meter)
  end
  self.crawler:Set({trackerName},opts)
end

function Settings:AliasString(subMeter)
  local aliases = {}
  for aliasName in self.handler:GetAll(subMeter, "alias") do
    if type(aliasName) == "string" or type(aliasName) == "number" then table.insert(aliases, tostring(aliasName)) end
  end
  if not D.empty(aliases) then
    return " (" .. table.concat(aliases, ", ")  .. ")"
  else return ""
  end
end

function Settings:BuildMeterSettings(trackerName, meterType, name, meter)
  if not meterTypes[meterType] then return end
  meter.newColor = meter.color
  local tableKey = K{ trackerName, "args", meterType, "args", name }
  self.crawler:Set(tableKey:r(2), {
    type = "group",
    name = meterTypes[meterType] .. "s",
    args = {}
  }, true)
  local meterSettings = {
    type = "group",
    name = name,
    args = {
      color = {
        type = "color",
        name = function() return name .. self:AliasString(meter) end,
        width ="full",
        order = 0
      },
      space0 = {
        type = "description",
        name = "",
        width = "full",
        order = 39
      },
      newAlso = {
        type = "input",
        name = "Add alternate",
        set = "AddAlso",
        order = 40
      },
      newColor = {
        type = "color",
        name = "",
        width = "half",
        order = 41
      },
      space1 = {
        type = "description",
        name = "",
        width = "full",
        order = 60
      },
      meterType = {
        type = "select",
        name = "Type",
        width = "half",
        values = meterTypes,
        order = 61
      },
      meterName = {
        type = "input",
        name = function() return "Include " .. (meter.meterType or "spell") end,
        set = "AddMeter",
        order = 62
      },
      delete = {
        type = "execute",
        name = "Delete",
        func = "DeleteMeter",
        order = 1012
      }
    },
    icon = function() return meter.image end
  }
  local i = 10
  for alsoName in self.handler:GetAll(meter,"also") do
    if alsoName ~= "" then
      meterSettings.args[alsoName .. "color"] = {
        type = "color",
        get = "ConfGetAlsoColor",
        set = "ConfSetAlsoColor",
        name = function() return alsoName .. self:AliasString(meter[alsoName]) end,
        order = i
      }
      meterSettings.args[alsoName .. "delete"] = {
        type = "execute",
        name = "x",
        width = "half",
        func = function(info)
          meterSettings.args[alsoName .. "color"] = nil
          meterSettings.args[alsoName .. "delete"] = nil
          self.handler:DeleteAlso(info)
        end,
        order = i + 1
      }
      meterSettings.args[alsoName .. "space"] = {
        type = "description",
        name = "",
        width = "full",
        order = i + 2
      }
      i = i + 3
    end
  end
  if meterType == "spell" then
    meterSettings.args["display"] = {
      type = "select",
      name = "Display",
      values = {
        normal = "Normal",
        individual = "Individual",
        aggregate = "Aggregate"
      },
      order = 51
    }
    meterSettings.args["markCrits"] = {
      type = "toggle",
      name = "Mark crits",
      order = 52
    }
  end
  self.crawler:Set(tableKey, meterSettings)
  for subType,subName,subMeter in self.handler:GetAll(meter, "subMeter", 3) do
    local entry = {
      type = "execute",
      name = function()
        return "× " .. meterTypes[subType] .. ": " .. subName .. self:AliasString(subMeter)
      end,
      width = "full",
      icon = function() return meter.image end
    }
    if subType == meterType and subName == name then
      entry.disabled = true
      entry.order = 90
      self.crawler:Set(tableKey:a("args", "root"), entry, true)
    else
      entry.type = "execute"
      entry.func = "DeleteSubMeter"
      local subName = subType .. "-" .. subName
      self.crawler:Set(tableKey:a("args", subName), entry, true)
    end
  end
end

function Settings:Added(template, name)
  if not browsing then return end
  return self.crawler:Get{ browsing, "args", template.type or "spell", "args", template.name or name }
end

function Settings:FormTemplate(from, name, template, noFunc)
  if not template.color then
    template.color = "000000"
  end
  local spell = {}
  local displayName = name
  for alsoName in pairs(template.also or {}) do
    spell = { GetSpellInfo(alsoName) }
    if spell[1] then
      displayName = spell[1]
      break
    end
  end
  if not spell[1] then spell = { GetSpellInfo(template.name or name) } end
  if not spell[1] then spell = { GetSpellInfo(template.id) } end
  local rainbows = self.handler:ConfGetDebug{ "rainbows" }
  local coloredName = rainbows and "|cff" .. template.color .. displayName .. "|r" or displayName
  if not template.type or template.type == "spell" or template.type == "aura" or template.type == "pet" then
    if template.pvp and not noFunc then from = self.fromPvp end
    from[name] = {
      type = "execute",
      name = coloredName,
      desc = GetSpellDescription(spell[7]),
      image = tostring(template.image or spell[3]),
      func = "AddFromTemplate",
      order = rainbows and D.colToHue(D.hexToCol(template.color)) or D.orderNum(name),
      disabled = function() return self:Added(template, name) end
    }
    if noFunc then
      from[name].func = nil
      from[name].disabled = nil
    end
  end
end

function Settings:UpdateSpec()
  for k in pairs(self.fromOther) do self.fromOther[k] = nil end
  for k in pairs(self.fromSpellbook) do self.fromSpellbook[k] = nil end
  for k in pairs(self.fromPvp) do self.fromPvp[k] = nil end
  local otherTemplates = Templates.all
  for name,template in pairs(otherTemplates or {}) do
    self:FormTemplate(self.fromOther, name, template)
  end
  local specTemplates = Templates[select(2,UnitClass("player"))][GetSpecialization()]
  if not specTemplates then return end
  for name,template in pairs(specTemplates or {}) do
    self:FormTemplate(self.fromSpellbook, name, template)
  end
end

function Settings:UpdateInv()
  for k in pairs(self.fromItem) do self.fromItem[k] = nil end
  for slot=1,19 do
    for _,item in pairs(GetInventoryItemsForSlot(slot,{}) or {}) do
      local template = Templates.items[item]
      if template and not template.color then
        template.color = "000000"
      end
      if template then self.fromItem[tostring(item)] = {
        type = "execute",
        name = self.handler:ConfGetDebug{ "rainbows" } and "|cff" .. template.color .. template.name .. "|r" or template.name,
        image = tostring(template.image or GetItemIcon(item)),
        func = "AddFromTemplate",
        order = D.orderNum(template.name),
        disabled = function() return self:Added(template) end
      } end
    end
  end
end


---------------
-- Debug panel
---------------


local function updateShowAll(info, val)
  if not info then
    val = Settings.handler:ConfGetDebug{ "showAll" }
  else
    Settings.handler:ConfSetDebug(info, val)
  end
  local rainbows = Settings.handler:ConfGetDebug{ "rainbows" }
  local allTemplates = Settings.debug.allTemplates
  allTemplates.args = {}
  if not val then
    allTemplates.hidden = true
    return
  end
  for className, class in pairs(Templates) do
    if className == "items" then
      allTemplates.args[className] = {
        type = "group",
        name = "Items",
        inline = false,
        args = {},
        order = 1000
      }
      local classTemplates = allTemplates.args[className]
      for itemID, template in pairs(class) do
        if not template.color then
          template.color = "000000"
        end
        classTemplates.args[itemID] = {
          type = "execute",
          name = rainbows and "|cff" .. template.color .. template.name .. "|r" or template.name,
          image = tostring(template.image or GetItemIcon(itemID)),
          order = rainbows and D.colToHue(D.hexToCol(template.color)) or D.orderNum(template.name)
        }
      end
    elseif className == "all" then
      allTemplates.args[className] = {
        type = "group",
        name = "Other",
        inline = false,
        args = {},
        order = 2000
      }
      local classTemplates = allTemplates.args[className]
      for name, template in pairs(class) do
        Settings:FormTemplate(classTemplates.args, name, template, true)
      end
    else
      local classID, classDisplay
      for i=1,GetNumClasses() do
        local subClassDisplay, subClassName, subClassID = GetClassInfo(i)
        if subClassName == className then
          classID = subClassID
          classDisplay = subClassDisplay
          break
        end
      end
      if classID then
        allTemplates.args[className] = {
          type = "group",
          name = classDisplay,
          inline = false,
          args = {}
        }
        local classTemplates = allTemplates.args[className]
        for specID, spec in pairs(class) do
          if specID > 0 then
            local specName = select(2, GetSpecializationInfoForClassID(classID, specID))
            classTemplates.args[specID] = {
              type = "group",
              name = specName,
              args = {}
            }
            local specTemplates = classTemplates.args[specID]
            for name,template in pairs(spec) do
              Settings:FormTemplate(specTemplates.args, name, template, true)
            end
          end
        end
      end
    end
  end
  allTemplates.hidden = false
end

Settings.debug = {
  rainbows = {
    type = "toggle",
    name = "Rainbows",
    set = function(info, val)
      Settings.handler:ConfSetDebug(info, val)
      Settings:UpdateInv()
      Settings:UpdateSpec()
      updateShowAll()
    end,
    width = "full",
    order = 10
  },
  showAll = {
    type = "toggle",
    name = "Show all templates below",
    set = updateShowAll,
    width = "full",
    order = 20
  },
  allTemplates = {
    type = "group",
    name = "All Templates",
    childGroups = "tab",
    hidden = true,
    inline = true,
    args = {},
    order = 100
  }
}